#! /usr/bin/env python
# -*- coding: utf-8 -*-
# filename: leica_tcr_705.py
# Copyright 2009 Luca Bianconi <luxetluc@yahoo.it>
# Copyright 2009,2011 Stefano Costa <steko@iosa.it>

# This file is part of Total Open Station.

# Total Open Station is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.

# Total Open Station is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with Total Open Station.  If not, see
# <http://www.gnu.org/licenses/>.


from . import *


class ModelConnector(Connector):

    def __init__(self, port):
        Connector.__init__(self, port=port, baudrate=19200)



import unittest

import pytest

from totalopenstation.formats.carlson_rw5 import FormatParser

from . import BaseTestOutput


class TestCarlsonRW5Parser(unittest.TestCase):
    def setUp(self):
        with open('sample_data/carlson_rw5/Leica1200.rw5') as testdata:
            fp = FormatParser(testdata.read())
            self.pts = list(fp.points)

    def test_point_xy(self):
        self.assertAlmostEqual(self.pts[0].geometry.x, 942130.662, places=3)
        self.assertAlmostEqual(self.pts[0].geometry.y, 16556174.237, places=3)

    @unittest.expectedFailure
    def test_point_z(self):
        self.assertAlmostEqual(self.pts[0].geometry.z, 20.053, places=3)

    def test_feature(self):
        self.assertEqual(self.pts[1].point_name, '108')
        self.assertEqual(self.pts[2].id, 2)
        self.assertEqual(self.pts[4].properties['attrib'][0], 'LIGHT POLE')
        self.assertEqual(self.pts[3].desc, 'PT')



class TestRW5Output(BaseTestOutput):

    @pytest.fixture
    def setup(self):
        with open('sample_data/carlson_rw5/Leica1200.rw5') as testdata:
            self.fp = FormatParser(testdata.read())



# -*- coding: utf-8 -*-
# filename: formats/leica_tcr_1205.py
# Copyright 2008-2009 Stefano Costa <steko@iosa.it>
# Copyright 2008 Luca Bianconi <luxetluc@yahoo.it>

# This file is part of Total Open Station.

# Total Open Station is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.

# Total Open Station is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with Total Open Station.  If not, see
# <http://www.gnu.org/licenses/>.

from . import Feature, Parser, Point


class FormatParser(Parser):

    def is_point(self, line):

        tokens = line.split()

        try:
            float(tokens[1])
            float(tokens[2])
            float(tokens[3])
        except (ValueError, IndexError):
            is_point = False
        else:
            #di questo controllo che segue FORSE non gliene frega un beliscimu
            if tokens[4] == "MEAS":
                is_point = True
            else:
                is_point = False
        return is_point

    def get_point(self, line):

        tokens = line.split()

        if len(tokens) > 5:
            text = str(tokens[5])
        else:
            text = ""

        p = Point(tokens[1],
                  tokens[2],
                  tokens[3])
        f = Feature(p,
                    desc=text,
                    id=tokens[0].strip())
        return f



## -*- coding: utf-8 -*-
# filename: formats/zeiss_r5.py
# Copyright 2015 Stefano Costa <steko@iosa.it>

# This file is part of Total Open Station.

# Total Open Station is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.

# Total Open Station is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with Total Open Station.  If not, see
# <http://www.gnu.org/licenses/>.

import logging

from . import Feature, Point

logger = logging.getLogger(__name__)


class FormatParser:

    def __init__(self, data):
        self.rows = (r for r in data.splitlines())

    @property
    def points(self):
        '''Point features.'''

        points = []

        def record(recstr):
            fields = recstr.split('|')
            record_fields = {}
            adr, reco, coords = fields[1], fields[2], fields[3:6]
            record_fields['Adr'] = adr.split()[1].strip()
            record_fields['type'] = reco[0:2]
            if record_fields['type'] == 'KR':
                record_fields['id'] = reco[6:10]
                record_fields['desc'] = reco[3:6]
                for xyz in coords:
                    coord = xyz[0]
                    value = xyz[1:].strip()
                    if value.endswith(' m'):
                        value = value[0:-2]
                    if len(value) > 2:
                        record_fields[coord] = value
            elif record_fields['type'] == 'TR':
                record_fields['desc'] = reco[3:10]
            logger.info("record_fields : %s" % (record_fields))
            return record_fields

        for row in self.rows:
            if row.startswith('END'):
                break
            else:
                rec = record(row)
                if rec['type'] == 'KR':
                    try:
                        point = Point(rec['X'],
                                      rec['Y'],
                                      rec['Z'])
                    except KeyError:
                        try:
                            point = Point(rec['X'],
                                          rec['Y'])
                        except KeyError:
                            continue
                    finally:
                        feature = Feature(point,
                                          desc=rec['desc'],
                                          id=rec['id'])
                        points.append(feature)
        logger.debug(points)
        return points



#! /usr/bin/env python
# -*- coding: utf-8 -*-
# filename: tops_sql.py
# Copyright 2019 Stefano Costa <steko@iosa.it>
#
# This file is part of Total Open Station.
#
# Total Open Station is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# Total Open Station is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Total Open Station.  If not, see
# <http://www.gnu.org/licenses/>.

from . import Builder


def to_sql(point, tablename):
    '''Generate SQL line corresponding to the input point.

    At this moment the column names are fixed, but they could change in the
    future. The default names are reasonable.'''

    params = {
        'wkt': to_wkt(point),
        'tablename': tablename,
        'pid': point.id,
        'text': point.desc}
    sql_string = "INSERT INTO %(tablename)s" % params
    sql_string += "(point_id, point_geom, point_text) VALUES"
    sql_string += "(%(pid)s,GeomFromText('%(wkt)s'),'%(text)s');\n" % params
    return sql_string


def to_wkt(point):
    x = point.geometry.x
    y = point.geometry.y
    try:
        point.geometry.z
    except ValueError:
        wkt_representation = 'POINT ({g.x} {g.y})'.format(g=point.geometry)
    else:
        wkt_representation = 'POINT Z ({g.x} {g.y} {g.z})'.format(g=point.geometry)
    return wkt_representation


class OutputFormat(Builder):

    """
    Exports points data in SQL format suitable for use with PostGIS & friends.

    http://postgis.refractions.net/documentation/manual-1.3/ch04.html#id2986280
    has an example of loading an SQL file into a PostgreSQL database.

    ``data`` must be an iterable containing Feature objects.
    """

    def __init__(self, data, tablename='topsdata'):
        self.data = data
        self.tablename = tablename

    def process(self):
        lines = [to_sql(e, self.tablename) for e in self.data]
        lines.insert(0, 'BEGIN;\n')
        lines.append('COMMIT;\n')
        output = "".join(lines)
        return output



# -*- coding: utf-8 -*-
# filename: formats/zeiss_rec_500.py
# Copyright 2008-2011 Stefano Costa <steko@iosa.it>
# Copyright 2008 Luca Bianconi <luxetluc@yahoo.it>

# This file is part of Total Open Station.

# Total Open Station is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.

# Total Open Station is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with Total Open Station.  If not, see
# <http://www.gnu.org/licenses/>.

from . import Feature, Parser, Point


class FormatParser(Parser):

    def is_point(self, line):
        try:
            tokens = {
                'sequence': line[0:7],
                'pid': line[8:27],
                'text': line[27:32],
                'X_str': line[36],
                'x': line[38:50],
                'Y_str': line[51],
                'y': line[53:66],
                'Z_str': line[67],
                'z': line[69:80],
                }
            int(tokens['sequence'])
            int(tokens['pid'])
            float(tokens['x'])
            float(tokens['y'])
            float(tokens['z'])
            assert tokens['X_str'] == 'X'
            assert tokens['Y_str'] == 'Y'
            assert tokens['Z_str'] == 'Z'
        except (ValueError, IndexError, AssertionError):
            is_point = False
        else:
            is_point = True
        return is_point

    def get_point(self, line):
        '''Gets a point from a line retrieving basic data.'''

        tokens = {
            'pid': line[8:27].strip(),   # the result is more elegant than
            'text': line[27:32].strip(), # the code (Heisenberg rocks!)
            'x': line[38:50].strip(),
            'y': line[53:66].strip(),
            'z': line[69:80].strip(),
            }

        point_id = int(tokens['pid'])
        text = str(tokens['text'])

        # note that for now we keep floats into strings to avoid approximation
        # problems, provided that for writing DXF a string is sufficient.
        # FIXME before introducing new output formats.
        # We could use string formatting operations to store data as floats
        # and convert them to strings with the needed precision on the fly.

        x = str(tokens['x'])
        y = str(tokens['y'])
        z = str(tokens['z'])

        # Even here it would have been better not giving x and y the
        # wrong values(the inverted ones)but directly the right ones!

        p = Point(y, x, z)
        f = Feature(p, id=point_id, desc=text)

        return f



from setuptools import setup, find_packages

import totalopenstation

setup(
    name='totalopenstation',
    version=totalopenstation.__version__,
    author='Stefano Costa',
    author_email='steko@iosa.it',
    packages=find_packages(exclude=['ez_setup', 'examples', 'tests', 'gui']),
    scripts=['scripts/totalopenstation-gui.py',
             'scripts/totalopenstation-cli-parser.py',
             'scripts/totalopenstation-cli-connector.py'],
    url='https://tops.iosa.it/',
    license='GNU GPLv3',
    description='Download and export survey data from your total station',
    long_description=open('README.rst').read(),
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Environment :: X11 Applications',
        'Intended Audience :: End Users/Desktop',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3 :: Only',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Topic :: Scientific/Engineering :: GIS',
        ],
    keywords='survey geodimeter',
    install_requires=['pyserial==3.4', 'pygeoif==0.7'],
    tests_require=['pytest>=5.1'],
    include_package_data = True,
    zip_safe = False,
)



# -*- coding: utf-8 -*-
# filename: formats/__init__.py
# Copyright 2008-2009 Luca Bianconi <luxetluc@yahoo.it>
# Copyright 2008-2011 Stefano Costa <steko@iosa.it>

# This file is part of Total Open Station.

# Total Open Station is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.

# Total Open Station is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with Total Open Station.  If not, see
# <http://www.gnu.org/licenses/>.


import serial
import sys

from time import sleep
from threading import Event, Thread

from totalopenstation.utils.upref import UserPrefs

class Connector(serial.Serial, Thread):
    '''Connect to a total station.

    For more information : `Pyserial documentation <https://pythonhosted.org/pyserial/pyserial_api.html#native-ports>`_

    Args:
        port: Device name or :const:`None`.
        baudrate (int): Baud rate such as 9600 or 115200 etc.
        bytesize: Number of data bits.
        parity: Enable parity checking.
        stopbits: Number of stop bits.
        timeout (float): Set a read timeout value.
        xonxoff (bool): Enable software flow control.
        rtscts (bool): Enable hardware (RTS/CTS) flow control.
        bool dsrdtr (bool): Enable hardware (DSR/DTR) flow control.
        writeTimeout (float): Set a write timeout value.
    '''

    def __init__(self, port=None, baudrate=9600, bytesize=8, parity='N',
                stopbits=1, timeout=None, xonxoff=0, rtscts=0,
                writeTimeout=None, dsrdtr=None):

        self.upref = UserPrefs()
        sleeptime = float(self.upref.getvalue('sleeptime'))

        Thread.__init__(self)
        self.dl_started = Event()
        self.dl_finished = Event()

        serial.Serial.__init__(self, port=port, baudrate=baudrate,
        bytesize=bytesize, parity=parity, stopbits=stopbits, timeout=timeout,
        xonxoff=xonxoff, rtscts=rtscts, writeTimeout=writeTimeout,
        dsrdtr=dsrdtr)

    def open(self):
        '''Open the serial link.
        '''

        serial.Serial.open(self)

    def download(self):
        '''Download method for user interfaces.

        First the class must be instantiated, then the port is open and the
        transfer from the device can start. Once the transfer is finished
        the user interface should call this method.'''

        n = self.inWaiting()
        result = self.read(n)

        # looks like there is a maximum buffer of 4096 characters, so we have
        # to wait for a short time and iterate the process until finished

        sleep(sleeptime)

        while self.inWaiting() > 0:
            result = result + self.read(self.inWaiting())
            sleep(sleeptime)

        self.result = result

    def fast_download(self):
        '''Implement a *fast* download method that requires less user input.

        Inside, it calls download() itself, just wrapping it inside another
        loop that checks whether there's input coming from the serial port:
        when data become to appear, download() can start.
        '''

        while self.inWaiting() == 0:
            sleep(sleeptime)
        self.dl_started.set()
        try:
            self.download()
        except threading.exceptions.KeyboardInterrupt:
            sys.exit()
        else:
            self.dl_finished.set()

    def run(self):
        self.fast_download()


BUILTIN_MODELS = {
    'leica_tcr_1205': ('leica_tcr_1205', 'ModelConnector', 'Leica TCR 1205'),
    'zeiss_elta_r55': ('zeiss_elta_r55', 'ModelConnector', 'Zeiss Elta R55'),
    'nikon_npl_350': ('nikon_npl_350', 'ModelConnector','Nikon NPL 350'),
    'leica_tcr_705': ('leica_tcr_705', 'ModelConnector', 'Leica TCR 705'),
    'trimble': ('trimble', 'ModelConnector', 'Trimble'),
    'custom': ('custom', 'CustomConnector', 'Custom/Unknown'),
    }



import unittest

import pytest

from totalopenstation.formats.sokkia_sdr33 import FormatParser

from . import BaseTestOutput


class TestSokkiaParser(unittest.TestCase):

    def setUp(self):
        with open('sample_data/sokkia_sdr33.tops') as testdata:
            self.fp = FormatParser(testdata.read())

    def test_point(self):
        self.assertAlmostEqual(self.fp.points[0].geometry.coords[0][1], 509.970)
        self.assertAlmostEqual(self.fp.points[0].geometry.y, 509.970)
        self.assertAlmostEqual(self.fp.points[0].geometry.coords[0][0], 937.274)
        self.assertAlmostEqual(self.fp.points[0].geometry.coords[0][2], 20.053)

    def test_feature(self):
        self.assertEqual(self.fp.points[0].id, 31)
        self.assertEqual(self.fp.points[0].desc, '11')
        self.assertEqual(self.fp.points[1].desc, '11')

    def test_linestring(self):
        self.ls = self.fp.build_linestring()
        self.assertAlmostEqual(self.ls.coords[0][0], 937.274)


class TestSokkiaOutput(BaseTestOutput):

    @pytest.fixture
    def setup(self):
        with open('sample_data/sokkia_sdr33.tops') as testdata:
            self.fp = FormatParser(testdata.read())



import unittest

import pytest

from totalopenstation.formats.zeiss_rec_500 import FormatParser

from . import BaseTestOutput


class TestZeissParser(unittest.TestCase):

    def setUp(self):
        testdata = '   0076                 576  A28    X       72.702 Y       -92.823 Z   156.620 '
        self.fp = FormatParser(testdata)

    def test_zeiss(self):
        self.assertTrue(self.fp.is_point(self.fp.data))
        self.assertEqual(self.fp.points[0].geometry.y, 72.702)
        self.assertEqual(self.fp.points[0].geometry.x, -92.823)
        self.assertEqual(len(self.fp.points), 1)

class TestZeissExceptionValue(unittest.TestCase):

    def setUp(self):
        testdata = '   0076             576  A28    X       72.702 Y       -92.823 Z   156.620 '
        self.fp = FormatParser(testdata)

    def test_exceptions(self):
        self.assertRaises(ValueError, self.fp.get_point, self.fp.data)


class TestZeissExceptionIndex(unittest.TestCase):

    def setUp(self):
        testdata = 'END'
        self.fp = FormatParser(testdata)

    def test_exceptions(self):
        self.assertRaises(ValueError, self.fp.get_point, self.fp.data)


class TestZeissRecOutput(BaseTestOutput):
    @pytest.fixture
    def setup(self):
        with open('sample_data/zeiss_elta_r55/zeiss_elta_r55-REC_500.tops') as testdata:
            self.fp = FormatParser(testdata.read())



#! /usr/bin/env python
# -*- coding: utf-8 -*-
# filename: totalopenstation-cli-parser.py
# Copyright 2008-2013 Stefano Costa <steko@iosa.it>
# Copyright 2015-2016 Damien Gaignon <damien.gaignon@gmail.com>

# This file is part of Total Open Station.

# Total Open Station is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.

# Total Open Station is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with Total Open Station.  If not, see
# <http://www.gnu.org/licenses/>.

import sys
import os
import gettext
import importlib

import logging

from optparse import OptionParser

import totalopenstation.formats
import totalopenstation.output


t = gettext.translation('totalopenstation', './locale', fallback=True)
_ = t.gettext

usage = _("usage: %prog [option] arg1 [option] arg2 ...")

parser = OptionParser(usage=usage)
parser.add_option("-i",
                "--infile",
                action="store",
                type="string",
                dest="infile",
                help=_("select input FILE  (do not specify for stdin)"),
                metavar="FILE")
parser.add_option("-o",
                "--outfile",
                action="store",
                type="string",
                dest="outfile",
                help=_("select output FILE (do not specify for stdout)"),
                metavar="FILE")
parser.add_option("-f",
                "--input-format",
                action="store",
                type="string",
                dest="informat",
                help=_("select input FORMAT"),
                metavar="FORMAT")
parser.add_option("--2d",
                  action="store_true",
                  dest="xy_only",
                  help=_("Exclude Z coordinates, output only 2D data"),
                  metavar="ONLY2D")
parser.add_option("-t",
                "--output-format",
                action="store",
                type="string",
                dest="outformat",
                help=_("select input FORMAT"),
                metavar="FORMAT")
parser.add_option("-r",
                "--raw",
                action="store_true",
                dest="raw",
                help=_("Enhanced parsed file process"))
parser.add_option(
                "--overwrite",
                action="store_true",
                dest="overwrite",
                default=False,
                help=_("overwrite existing output file"))
parser.add_option(
    "--list",
    action="store_true",
    dest="list",
    default=False,
    help=_("list the available input and output formats"))
parser.add_option(
                "--log",
                action="store",
                dest="loglevel",
                default="WARNING",
                type="string",
                help=_("minimum log level"))
parser.add_option(
                "--logtofile",
                action="store_true",
                dest="logotfile",
                default=False,
                help=_("log to file"))


(options, args) = parser.parse_args()

logger = logging.getLogger()
logger.setLevel(options.loglevel.upper())
if options.logotfile:
    handler = logging.FileHandler("tops.log")
else:
    handler = logging.StreamHandler()
formatter = logging.Formatter('%(asctime)s -- %(filename)s -- %(levelname)s -- %(funcName)s -- %(message)s')
handler.setFormatter(formatter)
logger.addHandler(handler)

def list_formats():
    '''Print a list of the supported input and output formats.'''

    mod_string = "List of supported input formats:\n" + "-" * 30 + "\n"
    for k, v in sorted(totalopenstation.formats.BUILTIN_INPUT_FORMATS.items()):
        mod_string += k.ljust(20) + v[2] + "\n"
    mod_string += "\n\n"

    mod_string += "List of supported output formats:\n" + "-" * 30 + "\n"
    for k, v in sorted(totalopenstation.output.BUILTIN_OUTPUT_FORMATS.items()):
        mod_string += k.ljust(20) + v[2] + "\n"
    mod_string += "\n"
    return mod_string

if options.list:
    sys.stdout.write(list_formats())
    sys.exit()

def exit_with_error(message):
    sys.exit(_("\nError:\n%(message)s\n\n%(formats)s") % {'message': message,
                                                          'formats': list_formats()})

if options.informat:
    try:
        inputclass = totalopenstation.formats.BUILTIN_INPUT_FORMATS[options.informat]
    except KeyError as message:
        exit_with_error(_('%s is not a valid input format') % message)
    else:
        if isinstance(inputclass, tuple):
            try:
                # builtin format parser
                mod, cls, name = inputclass
                inputclass = getattr(importlib.import_module('totalopenstation.formats.' + mod), cls)
            except ImportError as message:
                exit_with_error(message)
else:
    sys.exit(_("Please specify an input format"))

if options.outformat:
    try:
        outputclass = totalopenstation.output.BUILTIN_OUTPUT_FORMATS[options.outformat]
    except KeyError as message:
        exit_with_error('%s is not a valid output format' % message)
    else:
        if isinstance(outputclass, tuple):
            try:
                # builtin output builder
                mod, cls, name = outputclass
                outputclass = getattr(importlib.import_module('totalopenstation.output.' + mod), cls)
            except ImportError as message:
                exit_with_error(message)

if options.infile:
    infile = open(options.infile, 'r').read()
else:
    if sys.stdin.isatty():
        sys.exit(_('No input data!'))
    else:
        infile = sys.stdin.read()


def main(infile):
    '''After setting up all parameters, finally try to process input data.'''

    parsed_data = inputclass(infile)
    if options.raw:
        parsed_points = parsed_data.raw_line
    else:
        parsed_points = parsed_data.points

    # processing options
    if options.xy_only:
        for feature in parsed_points:
            geom_cls = getattr(totalopenstation.formats, feature.geometry.geom_type)
            try:
                feature.geometry = geom_cls(feature.geometry.x, feature.geometry.y)
            except:
                feature.geometry = geom_cls([(p.x, p.y) for p in feature.geometry.geoms])
    output = outputclass(parsed_points)

    def write_to_file(outfile):
        e = open(outfile, 'w')
        e.write(output.process())
        e.close()

    if options.outfile:
        if not os.path.exists(options.outfile):
            write_to_file(options.outfile)
            logger.info(_("Downloaded data saved to out file %s") % options.outfile)
        else:
            if options.overwrite:
                write_to_file(options.outfile)
                logger.info(_("Downloaded data saved to file %s,") % (options.outfile))
                logger.info(_("overwriting the existing file"))
            else:
                sys.exit(_("Specified output file already exists\n"))
    else:
        sys.stdout.write(output.process())

if __name__ == '__main__':
    main(infile)
